# coding: utf-8
# pylint: disable=function-redefined
from django.apps import AppConfig
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.db.backends.signals import connection_created

from educommon.utils.system import is_in_migration_command


class AppConfig(AppConfig):

    name = __package__

    @property
    def _dispatch_uid(self):
        return '.'.join((
            __package__,
            self.__class__.__name__,
            self._configure_audit_log.__name__,
        ))

    def _configure_audit_log(self, connection, **kwargs):
        u"""Настраивает AuditLog при первом подключении к БД."""
        # Отключение сигнала во избежание повторного срабатывания.
        connection_created.disconnect(
            receiver=self._configure_audit_log,
            dispatch_uid=self._dispatch_uid,
        )

        if connection.alias != settings.DEFAULT_DB_ALIAS:
            return

        # Настройка параметров подключения к сервисной БД, обновление триггеров
        from educommon.audit_log.utils import configure
        configure()

        # Проверка подключения подключения к сервисной БД через FDW.
        from educommon.audit_log.utils import check_connection_fdw
        success, error_message = check_connection_fdw()
        if not success:
            raise ImproperlyConfigured(
                u"{0} - Ошибка подключения к сервисной базе через "
                u"postgres_fdw. Необходимо убедится что 'Журнал изменений' "
                u"настроен корректно.".format(error_message)
            )

    def ready(self):
        from .utils import is_initialized

        if (
            not is_in_migration_command() and
            is_initialized(settings.DEFAULT_DB_ALIAS)
        ):
            # Подключение обработчика, настраивающего AuditLog
            connection_created.connect(
                receiver=self._configure_audit_log,
                dispatch_uid=self._dispatch_uid,
            )
